@@ -4,7 +4,7 @@ written by Michael Stroeder <michael@stroeder.com>
 
 See http://python-ldap.sourceforge.net for details.
 
-\$Id: async.py,v 1.21 2004/03/10 19:37:57 stroeder Exp $
+\$Id: async.py,v 1.24 2005/11/07 11:43:00 stroeder Exp $
 
 Python compability note:
 Tested on Python 2.0+ but should run on Python 1.5.x.
@@ -12,7 +12,7 @@ Tested on Python 2.0+ but should run on Python 1.5.x.
 
 import ldap
 
-__version__ = '0.0.3'
+__version__ = '0.1.0'
 
 
 _searchResultTypes={
@@ -65,7 +65,9 @@ class AsyncSearchHandler:
     attrList=None,
     attrsOnly=0,
     timeout=-1,
-    sizelimit=0
+    sizelimit=0,
+    serverctrls=None,
+    clientctrls=None
   ):
     """
     searchRoot
@@ -80,13 +82,17 @@ class AsyncSearchHandler:
         See parameter attrsonly of method LDAPObject.search()
     timeout
         Maximum time the server shall use for search operation
-    sizeLimit
+    sizelimit
         Maximum number of entries a server should return
         (request client-side limit)
+    serverctrls
+        list of server-side LDAP controls
+    clientctrls
+        list of client-side LDAP controls
     """
     self._msgId = self._l.search_ext(
       searchRoot,searchScope,filterStr,
-      attrList,attrsOnly,None,None,timeout,sizelimit
+      attrList,attrsOnly,serverctrls,clientctrls,timeout,sizelimit
     )
     return # startSearch()
 
@@ -122,7 +128,7 @@ class AsyncSearchHandler:
       result_type,result_list = None,None
       while go_ahead:
         while result_type is None and not result_list:
-          result_type,result_list = self._l.result(self._msgId,0,timeout)
+          result_type,result_list,result_msgid,result_serverctrls = self._l.result3(self._msgId,0,timeout)
         if not result_list:
           break
         if not _searchResultTypes.has_key(result_type):
@@ -175,6 +181,22 @@ class List(AsyncSearchHandler):
     self.allResults.append((resultType,resultItem))
 
 
+class Dict(AsyncSearchHandler):
+  """
+  Class for collecting all search results into a dictionary {dn:entry}
+  """
+
+  def __init__(self,l):
+    AsyncSearchHandler.__init__(self,l)
+    self.allEntries = {}
+
+  def _processSingleResult(self,resultType,resultItem):
+    if _entryResultTypes.has_key(resultType):
+      # Search continuations are ignored
+      dn,entry = resultItem    
+      self.allEntries[dn] = entry
+
+
 class FileWriter(AsyncSearchHandler):
   """
   Class for writing a stream of LDAP search results to a file object
