@@ -2,10 +2,11 @@
 
 /* 
  * LDAPObject - wrapper around an LDAP* context
- * $Id: LDAPObject.c,v 1.61 2004/11/09 01:13:41 stroeder Exp $
+ * $Id: LDAPObject.c,v 1.69 2005/06/14 17:49:13 stroeder Exp $
  */
 
 #include "Python.h"
+#include "patchlevel.h"
 
 #include <math.h>
 #include <limits.h>
@@ -13,6 +14,7 @@
 #include "errors.h"
 #include "constants.h"
 #include "LDAPObject.h"
+#include "ldapcontrol.h"
 #include "message.h"
 #include "options.h"
 
@@ -71,7 +73,7 @@ not_valid( LDAPObject* l ) {
 	return 1;
     }
 }
-  
+
 /* free a LDAPMod (complete or partially) allocated in Tuple_to_LDAPMod() */
 
 static void
@@ -331,15 +333,33 @@ l_ldap_unbind_ext( LDAPObject* self, PyObject* args )
 {
     PyObject *serverctrls = Py_None;
     PyObject *clientctrls = Py_None;
+    LDAPControl** server_ldcs = NULL;
+    LDAPControl** client_ldcs = NULL;
 
     int ldaperror;
 
     if (!PyArg_ParseTuple( args, "|OO", &serverctrls, &clientctrls)) return NULL;
     if (not_valid(self)) return NULL;
+
+    if (!PyNone_Check(serverctrls)) {
+        server_ldcs = List_to_LDAPControls(serverctrls);
+        if (server_ldcs == NULL)
+            return NULL;
+    }
+
+    if (!PyNone_Check(clientctrls)) {
+        client_ldcs = List_to_LDAPControls(clientctrls);
+        if (client_ldcs == NULL)
+            return NULL;
+    }
+
     LDAP_BEGIN_ALLOW_THREADS( self );
-    ldaperror = ldap_unbind_ext( self->ldap, NULL, NULL );
+    ldaperror = ldap_unbind_ext( self->ldap, server_ldcs, client_ldcs );
     LDAP_END_ALLOW_THREADS( self );
 
+    LDAPControl_List_DEL( server_ldcs );
+    LDAPControl_List_DEL( client_ldcs );
+
     if ( ldaperror!=LDAP_SUCCESS )
     	return LDAPerror( self->ldap, "ldap_unbind_ext" );
 
@@ -356,15 +376,33 @@ l_ldap_abandon_ext( LDAPObject* self, PyObject* args )
     int msgid;
     PyObject *serverctrls = Py_None;
     PyObject *clientctrls = Py_None;
+    LDAPControl** server_ldcs = NULL;
+    LDAPControl** client_ldcs = NULL;
 
     int ldaperror;
 
     if (!PyArg_ParseTuple( args, "i|OO", &msgid, &serverctrls, &clientctrls)) return NULL;
     if (not_valid(self)) return NULL;
+
+    if (!PyNone_Check(serverctrls)) {
+        server_ldcs = List_to_LDAPControls(serverctrls);
+        if (server_ldcs == NULL)
+            return NULL;
+    }
+
+    if (!PyNone_Check(clientctrls)) {
+        client_ldcs = List_to_LDAPControls(clientctrls);
+        if (client_ldcs == NULL)
+            return NULL;
+    }
+
     LDAP_BEGIN_ALLOW_THREADS( self );
-    ldaperror = ldap_abandon_ext( self->ldap, msgid, NULL, NULL );
+    ldaperror = ldap_abandon_ext( self->ldap, msgid, server_ldcs, client_ldcs );
     LDAP_END_ALLOW_THREADS( self );
 
+    LDAPControl_List_DEL( server_ldcs );
+    LDAPControl_List_DEL( client_ldcs );
+
     if ( ldaperror!=LDAP_SUCCESS )
     	return LDAPerror( self->ldap, "ldap_abandon_ext" );
 
@@ -381,6 +419,8 @@ l_ldap_add_ext( LDAPObject* self, PyObject *args )
     PyObject *modlist;
     PyObject *serverctrls = Py_None;
     PyObject *clientctrls = Py_None;
+    LDAPControl** server_ldcs = NULL;
+    LDAPControl** client_ldcs = NULL;
 
     int msgid;
     int ldaperror;
@@ -393,10 +433,24 @@ l_ldap_add_ext( LDAPObject* self, PyObject *args )
     if (mods == NULL)
 	return NULL;
 
+    if (!PyNone_Check(serverctrls)) {
+        server_ldcs = List_to_LDAPControls(serverctrls);
+        if (server_ldcs == NULL)
+            return NULL;
+    }
+
+    if (!PyNone_Check(clientctrls)) {
+        client_ldcs = List_to_LDAPControls(clientctrls);
+        if (client_ldcs == NULL)
+            return NULL;
+    }
+
     LDAP_BEGIN_ALLOW_THREADS( self );
-    ldaperror = ldap_add_ext( self->ldap, dn, mods, NULL, NULL, &msgid);
+    ldaperror = ldap_add_ext( self->ldap, dn, mods, server_ldcs, client_ldcs, &msgid);
     LDAP_END_ALLOW_THREADS( self );
     LDAPMods_DEL( mods );
+    LDAPControl_List_DEL( server_ldcs );
+    LDAPControl_List_DEL( client_ldcs );
 
     if ( ldaperror!=LDAP_SUCCESS )
     	return LDAPerror( self->ldap, "ldap_add_ext" );
@@ -415,6 +469,8 @@ l_ldap_simple_bind( LDAPObject* self, PyObject* args )
     int cred_len;
     PyObject *serverctrls = Py_None;
     PyObject *clientctrls = Py_None;
+    LDAPControl** server_ldcs = NULL;
+    LDAPControl** client_ldcs = NULL;
     struct berval cred;
 
     if (!PyArg_ParseTuple( args, "ss#|OO", &who, &cred.bv_val, &cred_len, &serverctrls, &clientctrls )) return NULL;
@@ -422,10 +478,25 @@ l_ldap_simple_bind( LDAPObject* self, PyObject* args )
 
     if (not_valid(self)) return NULL;
 
+    if (!PyNone_Check(serverctrls)) {
+        server_ldcs = List_to_LDAPControls(serverctrls);
+        if (server_ldcs == NULL)
+            return NULL;
+    }
+
+    if (!PyNone_Check(clientctrls)) {
+        client_ldcs = List_to_LDAPControls(clientctrls);
+        if (client_ldcs == NULL)
+            return NULL;
+    }
+
     LDAP_BEGIN_ALLOW_THREADS( self );
-    ldaperror = ldap_sasl_bind( self->ldap, who, LDAP_SASL_SIMPLE, &cred, NULL, NULL, &msgid);
+    ldaperror = ldap_sasl_bind( self->ldap, who, LDAP_SASL_SIMPLE, &cred, server_ldcs, client_ldcs, &msgid);
     LDAP_END_ALLOW_THREADS( self );
 
+    LDAPControl_List_DEL( server_ldcs );
+    LDAPControl_List_DEL( client_ldcs );
+
     if ( ldaperror!=LDAP_SUCCESS )
     	return LDAPerror( self->ldap, "ldap_simple_bind" );
 
@@ -565,6 +636,8 @@ l_ldap_sasl_interactive_bind_s( LDAPObject* self, PyObject* args )
 
     PyObject *serverctrls = Py_None;
     PyObject *clientctrls = Py_None;
+    LDAPControl** server_ldcs = NULL;
+    LDAPControl** client_ldcs = NULL;
 
     PyObject       *SASLObject = NULL;
     PyObject *mechanism = NULL;
@@ -573,11 +646,34 @@ l_ldap_sasl_interactive_bind_s( LDAPObject* self, PyObject* args )
     void *defaults;
     static unsigned sasl_flags = LDAP_SASL_QUIET;
 
+    /* 
+     * In Python 2.3+, a "I" format argument indicates that we're either converting
+     * the Python object into a long or an unsigned int. In versions prior to that,
+     * it will always convert to a long. Since the sasl_flags variable is an
+     * unsigned int, we need to use the "I" flag if we're running Python 2.3+ and a
+     * "i" otherwise. 
+     */
+#if (PY_MAJOR_VERSION == 2) && (PY_MINOR_VERSION < 3)
+    if (!PyArg_ParseTuple(args, "sOOOi", &who, &SASLObject, &serverctrls, &clientctrls, &sasl_flags ))
+#else
     if (!PyArg_ParseTuple(args, "sOOOI", &who, &SASLObject, &serverctrls, &clientctrls, &sasl_flags ))
+#endif
       return NULL;
 
     if (not_valid(self)) return NULL;
 
+    if (!PyNone_Check(serverctrls)) {
+        server_ldcs = List_to_LDAPControls(serverctrls);
+        if (server_ldcs == NULL)
+            return NULL;
+    }
+
+    if (!PyNone_Check(clientctrls)) {
+        client_ldcs = List_to_LDAPControls(clientctrls);
+        if (client_ldcs == NULL)
+            return NULL;
+    }
+
     /* now we extract the sasl mechanism from the SASL Object */
     mechanism = PyObject_GetAttrString(SASLObject, "mech");
     if (mechanism == NULL) return NULL;
@@ -593,11 +689,15 @@ l_ldap_sasl_interactive_bind_s( LDAPObject* self, PyObject* args )
     msgid = ldap_sasl_interactive_bind_s(self->ldap, 
 					 who, 
 					 c_mechanism, 
-					 NULL, 
-					 NULL,
+					 server_ldcs, 
+					 client_ldcs,
 					 sasl_flags, 
 					 py_ldap_sasl_interaction, 
 					 SASLObject);
+
+    LDAPControl_List_DEL( server_ldcs );
+    LDAPControl_List_DEL( client_ldcs );
+
     if (msgid != LDAP_SUCCESS)
     	return LDAPerror( self->ldap, "ldap_sasl_interactive_bind_s" );
     return PyInt_FromLong( msgid );
@@ -612,6 +712,8 @@ l_ldap_compare_ext( LDAPObject* self, PyObject *args )
     char *dn, *attr;
     PyObject *serverctrls = Py_None;
     PyObject *clientctrls = Py_None;
+    LDAPControl** server_ldcs = NULL;
+    LDAPControl** client_ldcs = NULL;
 
     int msgid;
     int ldaperror;
@@ -623,10 +725,25 @@ l_ldap_compare_ext( LDAPObject* self, PyObject *args )
 
     if (not_valid(self)) return NULL;
 
+    if (!PyNone_Check(serverctrls)) {
+        server_ldcs = List_to_LDAPControls(serverctrls);
+        if (server_ldcs == NULL)
+            return NULL;
+    }
+
+    if (!PyNone_Check(clientctrls)) {
+        client_ldcs = List_to_LDAPControls(clientctrls);
+        if (client_ldcs == NULL)
+            return NULL;
+    }
+
     LDAP_BEGIN_ALLOW_THREADS( self );
-    ldaperror = ldap_compare_ext( self->ldap, dn, attr, &value, NULL, NULL, &msgid );
+    ldaperror = ldap_compare_ext( self->ldap, dn, attr, &value, server_ldcs, client_ldcs, &msgid );
     LDAP_END_ALLOW_THREADS( self );
 
+    LDAPControl_List_DEL( server_ldcs );
+    LDAPControl_List_DEL( client_ldcs );
+
     if ( ldaperror!=LDAP_SUCCESS )
     	return LDAPerror( self->ldap, "ldap_compare_ext" );
 
@@ -642,16 +759,34 @@ l_ldap_delete_ext( LDAPObject* self, PyObject *args )
     char *dn;
     PyObject *serverctrls = Py_None;
     PyObject *clientctrls = Py_None;
+    LDAPControl** server_ldcs = NULL;
+    LDAPControl** client_ldcs = NULL;
 
     int msgid;
     int ldaperror;
 
     if (!PyArg_ParseTuple( args, "s|OO", &dn, &serverctrls, &clientctrls )) return NULL;
     if (not_valid(self)) return NULL;
+
+    if (!PyNone_Check(serverctrls)) {
+        server_ldcs = List_to_LDAPControls(serverctrls);
+        if (server_ldcs == NULL)
+            return NULL;
+    }
+
+    if (!PyNone_Check(clientctrls)) {
+        client_ldcs = List_to_LDAPControls(clientctrls);
+        if (client_ldcs == NULL)
+            return NULL;
+    }
+
     LDAP_BEGIN_ALLOW_THREADS( self );
-    ldaperror = ldap_delete_ext( self->ldap, dn, NULL, NULL, &msgid );
+    ldaperror = ldap_delete_ext( self->ldap, dn, server_ldcs, client_ldcs, &msgid );
     LDAP_END_ALLOW_THREADS( self );
 
+    LDAPControl_List_DEL( server_ldcs );
+    LDAPControl_List_DEL( client_ldcs );
+
     if ( ldaperror!=LDAP_SUCCESS )
     	return LDAPerror( self->ldap, "ldap_delete_ext" );
 
@@ -668,6 +803,8 @@ l_ldap_modify_ext( LDAPObject* self, PyObject *args )
     PyObject *modlist;
     PyObject *serverctrls = Py_None;
     PyObject *clientctrls = Py_None;
+    LDAPControl** server_ldcs = NULL;
+    LDAPControl** client_ldcs = NULL;
 
     int msgid;
     int ldaperror;
@@ -680,11 +817,25 @@ l_ldap_modify_ext( LDAPObject* self, PyObject *args )
     if (mods == NULL)
 	return NULL;
 
+    if (!PyNone_Check(serverctrls)) {
+        server_ldcs = List_to_LDAPControls(serverctrls);
+        if (server_ldcs == NULL)
+            return NULL;
+    }
+
+    if (!PyNone_Check(clientctrls)) {
+        client_ldcs = List_to_LDAPControls(clientctrls);
+        if (client_ldcs == NULL)
+            return NULL;
+    }
+
     LDAP_BEGIN_ALLOW_THREADS( self );
-    ldaperror = ldap_modify_ext( self->ldap, dn, mods, NULL, NULL, &msgid );
+    ldaperror = ldap_modify_ext( self->ldap, dn, mods, server_ldcs, client_ldcs, &msgid );
     LDAP_END_ALLOW_THREADS( self );
 
     LDAPMods_DEL( mods );
+    LDAPControl_List_DEL( server_ldcs );
+    LDAPControl_List_DEL( client_ldcs );
 
     if ( ldaperror!=LDAP_SUCCESS )
     	return LDAPerror( self->ldap, "ldap_modify_ext" );
@@ -703,6 +854,8 @@ l_ldap_rename( LDAPObject* self, PyObject *args )
     int delold = 1;
     PyObject *serverctrls = Py_None;
     PyObject *clientctrls = Py_None;
+    LDAPControl** server_ldcs = NULL;
+    LDAPControl** client_ldcs = NULL;
 
     int msgid;
     int ldaperror;
@@ -711,10 +864,25 @@ l_ldap_rename( LDAPObject* self, PyObject *args )
     	return NULL;
     if (not_valid(self)) return NULL;
 
+    if (!PyNone_Check(serverctrls)) {
+        server_ldcs = List_to_LDAPControls(serverctrls);
+        if (server_ldcs == NULL)
+            return NULL;
+    }
+
+    if (!PyNone_Check(clientctrls)) {
+        client_ldcs = List_to_LDAPControls(clientctrls);
+        if (client_ldcs == NULL)
+            return NULL;
+    }
+
     LDAP_BEGIN_ALLOW_THREADS( self );
-    ldaperror = ldap_rename( self->ldap, dn, newrdn, newSuperior, delold, NULL, NULL, &msgid );
+    ldaperror = ldap_rename( self->ldap, dn, newrdn, newSuperior, delold, server_ldcs, client_ldcs, &msgid );
     LDAP_END_ALLOW_THREADS( self );
 
+    LDAPControl_List_DEL( server_ldcs );
+    LDAPControl_List_DEL( client_ldcs );
+
     if ( ldaperror!=LDAP_SUCCESS )
     	return LDAPerror( self->ldap, "ldap_rename" );
 
@@ -722,10 +890,10 @@ l_ldap_rename( LDAPObject* self, PyObject *args )
 }
 
 
-/* ldap_result2 */
+/* ldap_result3 */
 
 static PyObject *
-l_ldap_result2( LDAPObject* self, PyObject *args )
+l_ldap_result3( LDAPObject* self, PyObject *args )
 {
     int msgid = LDAP_RES_ANY;
     int all = 1;
@@ -734,7 +902,7 @@ l_ldap_result2( LDAPObject* self, PyObject *args )
     struct timeval* tvp;
     int res_type;
     LDAPMessage *msg = NULL;
-    PyObject *result_str, *retval, *pmsg;
+    PyObject *result_str, *retval, *pmsg, *pyctrls = 0;
     int res_msgid = 0;
 
     if (!PyArg_ParseTuple( args, "|iid", &msgid, &all, &timeout ))
@@ -753,7 +921,7 @@ l_ldap_result2( LDAPObject* self, PyObject *args )
     LDAP_END_ALLOW_THREADS( self );
 
     if (res_type < 0)	/* LDAP or system error */
-    	return LDAPerror( self->ldap, "ldap_result2" );
+    	return LDAPerror( self->ldap, "ldap_result3" );
 
     if (res_type == 0) {
 	/* Polls return (None, None, None); timeouts raise an exception */
@@ -773,8 +941,10 @@ l_ldap_result2( LDAPObject* self, PyObject *args )
     else {
 	int result;
 	char **refs = NULL;
+        LDAPControl **serverctrls = 0;
 	LDAP_BEGIN_ALLOW_THREADS( self );
-	ldap_parse_result( self->ldap, msg, &result, NULL, NULL, &refs, NULL, 0 );
+	ldap_parse_result( self->ldap, msg, &result, NULL, NULL, &refs,
+                           &serverctrls, 0 );
 	LDAP_END_ALLOW_THREADS( self );
 
 	if (result != LDAP_SUCCESS) {		/* result error */
@@ -786,6 +956,14 @@ l_ldap_result2( LDAPObject* self, PyObject *args )
 		e = "ldap_parse_result";
 	    return LDAPerror( self->ldap, e );
 	}
+
+        if (!(pyctrls = LDAPControls_to_List(serverctrls))) {
+            int err = LDAP_NO_MEMORY;
+            ldap_set_option(self->ldap, LDAP_OPT_ERROR_NUMBER, &err);
+            return LDAPerror(self->ldap, "LDAPControls_to_List");
+        }
+        ldap_controls_free(serverctrls);
+
 	pmsg = LDAPmessage_to_python( self->ldap, msg );
     }
 
@@ -794,11 +972,13 @@ l_ldap_result2( LDAPObject* self, PyObject *args )
     if (pmsg == NULL) {
 	    retval = NULL;
     } else {
-	    retval = Py_BuildValue("(OOi)", result_str, pmsg, res_msgid);
+        retval = Py_BuildValue("(OOiO)", result_str, pmsg, res_msgid,
+			       pyctrls ? pyctrls : PyList_New(0));
 	if (pmsg != Py_None) {
         Py_DECREF(pmsg);
     }
     }
+    Py_XDECREF(pyctrls);
     Py_DECREF(result_str);
     return retval;
 }
@@ -818,6 +998,8 @@ l_ldap_search_ext( LDAPObject* self, PyObject* args )
 
     PyObject *serverctrls = Py_None;
     PyObject *clientctrls = Py_None;
+    LDAPControl** server_ldcs = NULL;
+    LDAPControl** client_ldcs = NULL;
 
     double timeout = -1.0;
     struct timeval tv;
@@ -843,12 +1025,26 @@ l_ldap_search_ext( LDAPObject* self, PyObject* args )
     	tvp = NULL;
     }
 
+    if (!PyNone_Check(serverctrls)) {
+        server_ldcs = List_to_LDAPControls(serverctrls);
+        if (server_ldcs == NULL)
+            return NULL;
+    }
+
+    if (!PyNone_Check(clientctrls)) {
+        client_ldcs = List_to_LDAPControls(clientctrls);
+        if (client_ldcs == NULL)
+            return NULL;
+    }
+
     LDAP_BEGIN_ALLOW_THREADS( self );
     ldaperror = ldap_search_ext( self->ldap, base, scope, filter, attrs, attrsonly,
-                             NULL, NULL, tvp, sizelimit, &msgid );
+                             server_ldcs, client_ldcs, tvp, sizelimit, &msgid );
     LDAP_END_ALLOW_THREADS( self );
 
     free_attrs( &attrs );
+    LDAPControl_List_DEL( server_ldcs );
+    LDAPControl_List_DEL( client_ldcs );
 
     if ( ldaperror!=LDAP_SUCCESS )
     	return LDAPerror( self->ldap, "ldap_search_ext" );
@@ -865,6 +1061,8 @@ l_ldap_whoami_s( LDAPObject* self, PyObject* args )
 {
     PyObject *serverctrls = Py_None;
     PyObject *clientctrls = Py_None;
+    LDAPControl** server_ldcs = NULL;
+    LDAPControl** client_ldcs = NULL;
 
     struct berval *bvalue = NULL;
 
@@ -874,10 +1072,25 @@ l_ldap_whoami_s( LDAPObject* self, PyObject* args )
 
     if (!PyArg_ParseTuple( args, "|OO", &serverctrls, &clientctrls)) return NULL;
 
+    if (!PyNone_Check(serverctrls)) {
+        server_ldcs = List_to_LDAPControls(serverctrls);
+        if (server_ldcs == NULL)
+            return NULL;
+    }
+
+    if (!PyNone_Check(clientctrls)) {
+        client_ldcs = List_to_LDAPControls(clientctrls);
+        if (client_ldcs == NULL)
+            return NULL;
+    }
+
     LDAP_BEGIN_ALLOW_THREADS( self );
-    ldaperror = ldap_whoami_s( self->ldap, &bvalue, NULL, NULL );
+    ldaperror = ldap_whoami_s( self->ldap, &bvalue, server_ldcs, client_ldcs );
     LDAP_END_ALLOW_THREADS( self );
 
+    LDAPControl_List_DEL( server_ldcs );
+    LDAPControl_List_DEL( client_ldcs );
+
     if ( ldaperror!=LDAP_SUCCESS )
     	return LDAPerror( self->ldap, "ldap_whoami_s" );
 
@@ -910,39 +1123,6 @@ l_ldap_start_tls_s( LDAPObject* self, PyObject* args )
 
 #endif
 
-/* ldap_manage_dsa_it */
-
-static PyObject*
-l_ldap_manage_dsa_it( LDAPObject* self, PyObject* args )
-{
-    int result, manageDSAit, critical;
-    LDAPControl c;
-    LDAPControl *ctrls[2];
-    ctrls[0] = &c;
-    ctrls[1] = NULL;
-
-    if (!PyArg_ParseTuple( args, "i|i", &manageDSAit, &critical )) return NULL;
-    if (not_valid(self)) return NULL;
-
-    if ( manageDSAit ) {
-        c.ldctl_oid = LDAP_CONTROL_MANAGEDSAIT;
-        c.ldctl_value.bv_val = NULL;
-        c.ldctl_value.bv_len = 0;
-        c.ldctl_iscritical = critical;
-        result = ldap_set_option( self->ldap, LDAP_OPT_SERVER_CONTROLS, ctrls );
-    } else {
-        result = ldap_set_option( self->ldap, LDAP_OPT_SERVER_CONTROLS, NULL );
-    }
-
-    if ( result != LDAP_SUCCESS ){
-	return LDAPerror( self->ldap, "ldap_manage_dsa_it" );
-    }
-
-    Py_INCREF(Py_None);
-    return Py_None;
-}
-
-
 /* ldap_set_option */
 
 static PyObject*
@@ -975,6 +1155,8 @@ l_ldap_get_option(PyObject* self, PyObject *args)
 
 /* ldap_passwd */
 
+#if LDAP_VENDOR_VERSION>=20100
+
 static PyObject *
 l_ldap_passwd( LDAPObject* self, PyObject *args )
 {
@@ -986,6 +1168,8 @@ l_ldap_passwd( LDAPObject* self, PyObject *args )
     int newpw_len;
     PyObject *serverctrls = Py_None;
     PyObject *clientctrls = Py_None;
+    LDAPControl** server_ldcs = NULL;
+    LDAPControl** client_ldcs = NULL;
 
     int msgid;
     int ldaperror;
@@ -999,9 +1183,24 @@ l_ldap_passwd( LDAPObject* self, PyObject *args )
     
     if (not_valid(self)) return NULL;
 
+    if (!PyNone_Check(serverctrls)) {
+        server_ldcs = List_to_LDAPControls(serverctrls);
+        if (server_ldcs == NULL)
+            return NULL;
+    }
+
+    if (!PyNone_Check(clientctrls)) {
+        client_ldcs = List_to_LDAPControls(clientctrls);
+        if (client_ldcs == NULL)
+            return NULL;
+    }
+
     LDAP_BEGIN_ALLOW_THREADS( self );
-    ldaperror = ldap_passwd( self->ldap, &user, &oldpw, &newpw, NULL, NULL, &msgid );
+    ldaperror = ldap_passwd( self->ldap, &user, &oldpw, &newpw, server_ldcs, client_ldcs, &msgid );
     LDAP_END_ALLOW_THREADS( self );
+    
+    LDAPControl_List_DEL( server_ldcs );
+    LDAPControl_List_DEL( client_ldcs );
 
     if ( ldaperror!=LDAP_SUCCESS )
     	return LDAPerror( self->ldap, "ldap_passwd" );
@@ -1009,6 +1208,7 @@ l_ldap_passwd( LDAPObject* self, PyObject *args )
     return PyInt_FromLong( msgid );
 }
 
+#endif
 
 /* methods */
 
@@ -1024,7 +1224,7 @@ static PyMethodDef methods[] = {
     {"delete_ext",	(PyCFunction)l_ldap_delete_ext,		METH_VARARGS },
     {"modify_ext",	(PyCFunction)l_ldap_modify_ext,		METH_VARARGS },
     {"rename",	        (PyCFunction)l_ldap_rename,		METH_VARARGS },
-    {"result2",		(PyCFunction)l_ldap_result2,		METH_VARARGS },
+    {"result3",		(PyCFunction)l_ldap_result3,		METH_VARARGS },
     {"search_ext",	(PyCFunction)l_ldap_search_ext,		METH_VARARGS },
 #ifdef HAVE_TLS
     {"start_tls_s",	(PyCFunction)l_ldap_start_tls_s,	METH_VARARGS },
@@ -1035,7 +1235,6 @@ static PyMethodDef methods[] = {
 #if LDAP_VENDOR_VERSION>=20100
     {"passwd",	        (PyCFunction)l_ldap_passwd,	        METH_VARARGS },
 #endif
-    {"manage_dsa_it",	(PyCFunction)l_ldap_manage_dsa_it,	METH_VARARGS },
     {"set_option",	(PyCFunction)l_ldap_set_option,		METH_VARARGS },
     {"get_option",	(PyCFunction)l_ldap_get_option,		METH_VARARGS },
 #if defined(FILENO_SUPPORTED)
